#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var shelljs_1 = require("shelljs");
var semver = require("semver");
var fs_1 = require("fs");
var util_1 = require("util");
var common_1 = require("./common");
var path_1 = require("path");
var readline_1 = require("readline");
var chalk_1 = require("chalk");
function cleanup() {
    common_1.log('Cleaning up...');
    process.chdir(rootDir);
    shelljs_1.rm('-rf', tmpDir);
}
function printUsage() {
    shelljs_1.echo("Usage: intern-dev-release [help] [b=branch] [v=version] [p=prerelease]\n");
    shelljs_1.echo('\n');
    shelljs_1.echo("  help        Displays this message\n");
    shelljs_1.echo("  branch      Branch to release; defaults to the current branch.\n");
    shelljs_1.echo("  version     Version to release; defaults to what is listed in the\n");
    shelljs_1.echo("              package.json in the branch. It should only be specified\n");
    shelljs_1.echo("              for pre-releases\n");
    shelljs_1.echo("  prerelease  A prerelease tag to attach to the version, like 'alpha'\n");
    shelljs_1.echo("              or 'beta'.\n");
}
function prompt() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return __awaiter(this, void 0, void 0, function () {
        var question;
        return __generator(this, function (_a) {
            question = util_1.format.apply(void 0, [args[0]].concat(args.slice(1)));
            return [2, new Promise(function (resolve) {
                    rl.question(question, resolve);
                })];
        });
    });
}
function loadPackageJson() {
    return JSON.parse(fs_1.readFileSync('package.json', { encoding: 'utf8' }));
}
function updatePackageVersion(version) {
    var packageJson = loadPackageJson();
    packageJson.version = version;
    fs_1.writeFileSync('package.json', JSON.stringify(packageJson, null, '  '));
}
var rl = readline_1.createInterface({
    input: process.stdin,
    output: process.stdout
});
var branch = common_1.exec('git rev-parse --abbrev-ref HEAD').stdout.replace(/\s+$/, '');
var version;
var npmTag;
var preVersion;
var preTag;
var newBranch;
var branchVersion;
process.argv.slice(2).forEach(function (arg) {
    if (arg === 'help') {
        printUsage();
        process.exit(0);
    }
    var _a = arg.split('=', 2), key = _a[0], value = _a[1];
    switch (key) {
        case 'b':
            branch = value;
            break;
        case 'v':
            version = value;
            break;
        case 't':
            npmTag = value;
            break;
        case 'p':
            preTag = value;
            break;
        default:
            shelljs_1.echo("Invalid argument \"" + arg + "\"\n\n");
            printUsage();
            process.exit(1);
    }
});
var rootDir = process.cwd();
var tmpDir = '_publish';
var exitCode = 0;
var pushBranches = [branch];
if (!npmTag) {
    if (preTag || version) {
        npmTag = 'next';
    }
    else {
        npmTag = 'latest';
    }
}
(function main() {
    return __awaiter(this, void 0, void 0, function () {
        var question_1, output, currentBranch, message, packageJson, tagLines, tags_1, sameVersionTags, tags, publishDir, question, error_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 4, 5, 6]);
                    if (!(branch !== 'master')) return [3, 2];
                    question_1 = "Are you sure you want to create a release from branch " + branch + "?\n" +
                        "Enter \"y\" to continue, any other key to abort.\n" +
                        '> ';
                    return [4, prompt(question_1)];
                case 1:
                    if ((_a.sent()) !== 'y') {
                        throw new Error('Aborted');
                    }
                    _a.label = 2;
                case 2:
                    output = common_1.exec('git config receive.denyCurrentBranch').stdout;
                    if (output.indexOf('updateInstead') !== 0) {
                        throw new Error('Repository should have receive.denyCurrentBranch set to "updateInstead"');
                    }
                    currentBranch = common_1.exec('git rev-parse --abbrev-ref HEAD').stdout.replace(/\s+$/, '');
                    if (branch === currentBranch) {
                        try {
                            common_1.exec('git diff-index --quiet HEAD');
                        }
                        catch (error) {
                            common_1.log(chalk_1.red('Warning: You have uncommitted changes.'));
                        }
                    }
                    message = "Creating a new release from branch " + branch;
                    if (version) {
                        message += " with version override " + version;
                    }
                    common_1.log(message + ".");
                    process.chdir(rootDir);
                    if (shelljs_1.test('-d', tmpDir)) {
                        cleanup();
                    }
                    shelljs_1.mkdir(tmpDir);
                    common_1.exec("git clone --recursive . " + tmpDir);
                    process.chdir(tmpDir);
                    common_1.log("Building branch \"" + branch + "\"...");
                    common_1.exec("git checkout " + branch);
                    packageJson = loadPackageJson();
                    if (!version) {
                        version = packageJson.version;
                        if (!semver.prerelease(version)) {
                            throw new Error('Releases may only be generated from pre-release versions');
                        }
                        version = semver.major(version) + "." + semver.minor(version) + "." + semver.patch(version);
                        if (preTag) {
                            version += "-" + preTag;
                            tagLines = common_1.exec('git show-ref --tags --abbrev').stdout.replace(/\s+$/, '').split('\n');
                            tags_1 = tagLines.map(function (line) { return /refs\/tags\/(.*)/.exec(line)[1]; });
                            sameVersionTags = tags_1.filter(function (tag) {
                                try {
                                    return semver.major(tag) === semver.major(version) &&
                                        semver.minor(tag) === semver.minor(version) &&
                                        semver.patch(tag) === semver.patch(version) &&
                                        semver.prerelease(tag)[0] === preTag;
                                }
                                catch (error) {
                                    return false;
                                }
                            });
                            sameVersionTags.sort(function (a, b) {
                                var preA = Number(semver.prerelease(a)[1]);
                                var preB = Number(semver.prerelease(b)[1]);
                                return preB - preA;
                            });
                            version = semver.inc(sameVersionTags[0] || version, 'prerelease', preTag);
                        }
                    }
                    else {
                        if (semver.gte(packageJson.version, version)) {
                            throw new Error('Provided version must be >= current version');
                        }
                    }
                    tags = common_1.exec('git tag').stdout;
                    tags.split('\n').forEach(function (tag) {
                        if (tag === version) {
                            throw new Error('Version ' + tag + ' has already been tagged');
                        }
                    });
                    if (semver.major(version) === 0 || semver.patch(version) !== 0 || semver.prerelease(version)) {
                        preVersion = semver.inc(version, 'patch') + '-pre';
                    }
                    else {
                        newBranch = semver.major(version) + "." + semver.minor(version);
                        branchVersion = semver.inc(version, 'patch') + '-pre';
                        preVersion = semver.inc(version, 'minor') + '-pre';
                    }
                    updatePackageVersion(version);
                    common_1.exec("git commit -m \"Updating metadata for " + version + "\" package.json");
                    common_1.exec("git tag -a -m \"Release " + version + "\" " + version);
                    common_1.exec('git checkout HEAD^ package.json');
                    common_1.exec('git reset package.json');
                    updatePackageVersion(preVersion);
                    common_1.exec("git commit -m \"Updating source version to " + preVersion + "\" package.json");
                    if (newBranch) {
                        common_1.log("Creating new branch " + newBranch + "...");
                        common_1.exec("git checkout -b " + newBranch + " " + version);
                        updatePackageVersion(branchVersion);
                        common_1.exec("git commit -m \"Updating source version to " + branchVersion + "\" package.json");
                        pushBranches.push(newBranch);
                    }
                    common_1.log("Checking out and building " + version + "...");
                    common_1.exec("git checkout " + version);
                    common_1.exec('npm install');
                    common_1.exec('npm run build');
                    common_1.log('Done!');
                    publishDir = common_1.internDev && common_1.internDev.publishDir;
                    if (!publishDir) {
                        publishDir = common_1.buildDir;
                        if (fs_1.existsSync(path_1.join(common_1.buildDir, 'src'))) {
                            publishDir = path_1.join(common_1.buildDir, 'src');
                        }
                    }
                    common_1.log("Package to be published from " + tmpDir + "/" + publishDir + ".");
                    question = 'Please confirm packaging success, then enter "y" to publish to npm\n' +
                        ("'" + npmTag + "', push tag '" + version + "', and upload. Enter any other key to bail.\n") +
                        '> ';
                    return [4, prompt(question)];
                case 3:
                    if ((_a.sent()) !== 'y') {
                        common_1.log('Not publishing');
                        throw new Error('Aborted');
                    }
                    process.chdir(publishDir);
                    common_1.exec("npm publish --tag " + npmTag);
                    pushBranches.map(function (branch) {
                        common_1.exec("git push origin " + branch);
                    });
                    common_1.exec('git push origin --tags');
                    common_1.log('All done! Yay!');
                    return [3, 6];
                case 4:
                    error_1 = _a.sent();
                    if (error_1.message !== 'Aborted') {
                        common_1.log("" + chalk_1.red(error_1.stack));
                        common_1.log('Aborted.');
                        exitCode = 1;
                    }
                    return [3, 6];
                case 5:
                    if (exitCode !== 1) {
                        cleanup();
                    }
                    process.exit(exitCode);
                    return [7];
                case 6: return [2];
            }
        });
    });
})();
